import pygame

from math import ceil

from pygame.locals import *
from util import *

TYPE_TEXT = 0
TYPE_IMAGE = 1

ALIGN_LEFT = 0
ALIGN_CENTER = 1
ALIGN_RIGHT = 2

class menuitem():
    def __init__(self, selectable=False, selectid=None):
        self.itemtype = None
        self.selectable = selectable
        self.selectid = selectid

class menuimage(menuitem):
    def __init__(self, imagefile, selectable=False, selectid=None):
        menuitem.__init__(self, selectable, selectid)
        self.itemtype = TYPE_IMAGE
        self.imagefile = imagefile
        self.image = self.render()

    def render(self):
        fullname = os.path.join('img', self.imagefile)
        image = pygame.image.load(fullname)
        image = image.convert()
        colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
        return image

class menutext(menuitem):
    def __init__(self, text, font, color, selectable=False, selectid=None):
        menuitem.__init__(self, selectable, selectid)
        self.itemtype = TYPE_TEXT
        self.text = text
        self.image = self.render(font, color)

    def render(self, font, color):
        return font.render(self.text, False, color)

class menuline():
    def __init__(self, dimensions=(640, 480), align=ALIGN_LEFT):
        self.menuitems = []
        self.align = align
        self.selectdict = {}
        self.numselect = 0

    def initimage(self, dimensions):
        self.dimensions = dimensions
        self.image = pygame.Surface(dimensions)
        self.clearimage()

    def clearimage(self):
        self.image.fill((248, 248, 248))
        self.image.set_colorkey((248, 248, 248))
        
    def addtext(self, text, font, color, selectable=False, selectid=None):
        self.menuitems.append(menutext(text, font, color, selectable, selectid))

    def addimage(self, image, selectable=None, selectid=None):
        self.menuitems.append(menuimage(image, selectable, selectid))

    def buildselectdict(self, cellpadding):
        xoffset = 0
        for i in self.menuitems:
            if (i.selectable):
                self.selectdict[i.selectid] =(xoffset, 0)
                self.numselect += 1
            xoffset += i.image.get_width()
            xoffset += cellpadding

    def render(self, cellpadding):
        xoffset = 0
        width = 0
        maxheight = 0
        for i in self.menuitems:
            if (maxheight < i.image.get_height()):
                maxheight = i.image.get_height()
            width += i.image.get_width()
            width += cellpadding
        self.initimage((width, maxheight))
        for i in self.menuitems:
            self.image.blit(i.image, (xoffset, 0))
            xoffset += i.image.get_width()
            xoffset += cellpadding
        self.buildselectdict(cellpadding)        

class menulayout():
    def __init__(self, dimensions=(640, 480)):
        self.initimage(dimensions)
        self.menulines = []
        self.selectdict = {}
        self.numselect = 0

    def initimage(self, dimensions):
        self.dimensions = dimensions
        self.image = pygame.Surface(dimensions)
        self.clearimage()

    def clearimage(self):
        self.image.fill((248, 248, 248))
        self.image.set_colorkey((248, 248, 248))

    def addline(self, line):
        self.menulines.append(line)

    def buildselectdict(self, cellpadding):
        yoffset = 0
        for i in self.menulines:
            for key, coord in i.selectdict.iteritems():
                x, y = coord
                y = yoffset
                self.selectdict[key] = (x, y)
                self.numselect += 1
            yoffset += i.image.get_height()
            yoffset += cellpadding

    def render(self, cellpadding):
        self.clearimage()
        yoffset = 0
        for i in self.menulines:
            i.render(cellpadding)
            self.image.blit(i.image, (0, yoffset))
            yoffset += i.image.get_height() 
            yoffset += cellpadding
        self.buildselectdict(cellpadding)


class menu(pygame.sprite.Sprite):
    def __init__(self, menustyle, rect, menulayout, innerpadding, cellpadding):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
        self.image = pygame.Surface((rect.width, rect.height))
        self.rect = rect
        self.alive = True
        self.layout = menulayout
        self.borderimage, self.borderrect = load_image(menustyle)
        self.bordersize = self.borderrect.height / 3
        self.innerpadding = innerpadding
        self.cellpadding = cellpadding
        self.hidden = True
        self.initimage()

    def initimage(self):
        self.dimensions = (self.rect.width, self.rect.height)
        self.image = pygame.Surface(self.dimensions)
        self.clearimage()

    def clearimage(self):
        self.image.fill((248, 248, 248))
        self.image.set_colorkey((248, 248, 248))

    def isalive(self):
        return self.alive

    def getnextitem(self, selectid):
        if (selectid + 1) in self.layout.selectdict:
            return selectid + 1
        elif 0 in self.layout.selectdict:
            return 0
        else:
            return False

    def getpreviousitem(self, selectid):
        if (selectid - 1) in self.layout.selectdict:
            return selectid - 1
        elif (self.layout.numselect - 1) in self.layout.selectdict:
            return self.layout.numselect - 1
        else:
            return False

    def getcursorposition(self, selectid):
        x, y = self.layout.selectdict[selectid]
        return (x+self.rect.left, y+self.rect.top)
    
    def drawframe(self):
        numxcells = int(ceil(self.rect.width / self.bordersize))
        numycells = int(ceil(self.rect.height / self.bordersize))
        for i in range(numycells+1):
            for j in range(numxcells+1):
                area = False
                destination = False
                if (i == 0):
                    if (j == 0):
                         area = pygame.Rect(0, 0, self.bordersize, self.bordersize)
                         destination = (0, 0)
                    elif (j == numxcells):
                        area = pygame.Rect(self.bordersize * 2, 0, self.bordersize, self.bordersize)
                        destination = (self.rect.width - self.bordersize, 0)
                    else:
                        area = pygame.Rect(self.bordersize, 0, self.bordersize, self.bordersize)
                        destination = (self.bordersize * j, 0)
                elif (i == numycells):
                    if (j == 0):
                         area = pygame.Rect(0, self.bordersize * 2, self.bordersize, self.bordersize)
                         destination = (0, self.rect.height - self.bordersize)
                    elif (j == numxcells):
                        area = pygame.Rect(self.bordersize * 2, self.bordersize * 2, self.bordersize, self.bordersize)
                        destination = (self.rect.width - self.bordersize, self.rect.height - self.bordersize)
                    else:
                        area = pygame.Rect(self.bordersize, self.bordersize * 2, self.bordersize, self.bordersize)
                        destination = (self.bordersize * j, self.rect.height - self.bordersize)
                else:
                    if (j == 0):
                         area = pygame.Rect(0, self.bordersize, self.bordersize, self.bordersize)
                         destination = (0, self.bordersize * i)
                    elif (j == numxcells):
                        area = pygame.Rect(self.bordersize*2, self.bordersize, self.bordersize, self.bordersize)
                        destination = (self.rect.width - self.bordersize, self.bordersize * i)
                    else:
                        area = pygame.Rect(self.bordersize, self.bordersize, self.bordersize, self.bordersize)
                        destination = (self.bordersize * j, self.bordersize * i)
                if (destination):
                    self.image.blit(self.borderimage, destination, area)

    def render(self):
        self.drawframe()
        self.layout.render(self.cellpadding)
        self.image.blit(self.layout.image, (self.innerpadding,self.innerpadding))
        self.hidden = False

    def hide(self):
        self.hidden = True

    def setlayout(self, newlayout):
        self.layout = newlayout

class selectcursor(pygame.sprite.Sprite):
    def __init__(self, imagefile, selectid, xoffset, yoffset):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
        self.image, self.rect = load_image(imagefile, -1)
        self.selectid = selectid
        self.xoffset = xoffset
        self.yoffset = yoffset

    def move(self, coord):
        x, y = coord
        self.rect.top = y + self.yoffset
        self.rect.left = x + self.xoffset

class dialogscreen():
    def __init__(self, menu, cursor, screen):
        self.screen = screen
        
        self.menu = menu
        self.cursor = cursor
        self.cursor.move(self.menu.getcursorposition(self.cursor.selectid))
        self.menugroup = pygame.sprite.RenderPlain((self.mymenu, self.myselectcursor))
